/*
 COPYRIGHT 2012 ESRI
 
 TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
 Unpublished material - all rights reserved under the
 Copyright Laws of the United States and applicable international
 laws, treaties, and conventions.
 
 For additional information, contact:
 Environmental Systems Research Institute, Inc.
 Attn: Contracts and Legal Services Department
 380 New York Street
 Redlands, California, 92373
 USA
 
 email: contracts@esri.com
 */

@class AGSCredential;
@class AGSLayer;
@class AGSDynamicLayer;
@class AGSWMSLayerInfo;
@class AGSEnvelope;
@class AGSSpatialReference;
@protocol AGSSecuredResource;

/** @file AGSWMSLayer.h */ //Required for Globals API doc

#pragma mark -

/** @brief A layer based on a dynamic WMS Server map service.
 
 An instance of this class allows you to display maps from a WMS map service.
 This class searches for a spatial reference in the root layer.
 If it cannot find one there it searches each layer contained in the root layer.
 If no spatial reference is found the layer fails.
 For example, <a href=http://lasigpublic.nerc-lancaster.ac.uk/ArcGIS/services/Biodiversity/CSHedges/MapServer/WMSServer</a>.
 A dynamic map service generates map images on the fly.
 
 In a Model-View-Controller architecture, this object represents the Model. The 
 corresponding View object, @c AGSDynamicLayerView,  is created when this layer 
 is added to the map.
 
 @define{AGSWMSLayer.h,ArcGIS}
 @since 2.2
 */
@interface AGSWMSLayer : AGSDynamicLayer <AGSSecuredResource>

/** The version of the layer.
 @since 2.2
 */
@property (nonatomic, copy, readonly) NSString *version;

/** The copyright info for the layer.
 @since 2.2
*/
@property (nonatomic, copy, readonly) NSString *copyright;

/** The spatial reference of the layer,
 must be a valid EPSG code.
 @since 2.2
*/
@property (nonatomic, strong, readonly) AGSSpatialReference *spatialReference;


/** A list of @c AGSWMSLayerInfo objects, which contain the name, title, and extent
 of each subLayer.
 @since 2.2
 */
@property (nonatomic, copy, readonly) NSArray *layerInfos;


/** The credential to be used to access this layer's secured map service.
 @since 2.2
 */
@property (nonatomic, copy, readwrite) AGSCredential *credential;

/** The credential cache to be used for this resource. By default this will be set to the global cache.
 @since 10.1.1
 */
@property (nonatomic, strong, readwrite) AGSCredentialCache *credentialCache;

/** The image format of the map. Possible values include
 
 @li @c AGSImageFormatPNG32
 @li @c AGSImageFormatPNG24
 @li @c AGSImageFormatPNG8
 @li @c AGSImageFormatPNG
 @li @c AGSImageFormatJPG
 @li @c AGSImageFormatGIF 
 
 Please ensure the map service supports 
 the requested image format.
 @since 2.2
 */
@property (nonatomic) AGSImageFormat imageFormat;

/** A property indicating whether or not images generated by this layer's map service 
 should be transparent.
 @since 2.2
 */
@property (nonatomic) BOOL transparent;

/** The sub-layers to be included in the map image exported by the map service. 
 By default the array is nil.
 To exclude certain layers from the image, you need to set this property to
 an array containing the names of each @c AGSWMSLayerInfo that needs to
 be included in the image. The layers that are left out will not be included 
 in the image.
 @since 2.2
 */
@property (nonatomic, copy) NSArray *visibleLayers;


/** Initialize this layer with a URL of a WMS Server dynamic map service.
 @param url URL to a WMS dynamic map service.
 @return A new dynamic WMS layer object.
 @since 2.2
 */
- (id)initWithURL:(NSURL *)url;

/** Initialize this layer with a URL of a WMS Server dynamic map service as well as a
 credential to a secured resource.
 @param url URL to a WMS dynamic map service.
 @param cred @c AGSCredential to access a secured resource.
 @return A new dynamic WMS layer object.
 @since 2.2
 */
- (id)initWithURL:(NSURL *)url credential:(AGSCredential *)cred;

/** A method to get an initialized, autoreleased layer with a URL of a WMS map service.
 @param url URL to a WMS map service.
 @return A new, autoreleased, WMS map service layer object.
 @since 2.2
 */
+ (id)wmsLayerWithURL:(NSURL *)url;

/** A method to get an initialized, autoreleased layer with a URL of a WMS map service.
 @param url URL to WMS map service.
 @param cred @c AGSCredential used to access a secured resource.
 @return A new, autoreleased, WMS map service layer object.
 @since 2.2
 */
+ (id)wmsLayerWithURL:(NSURL *)url credential:(AGSCredential*)cred;


@end
